library(MASS)

do.it <- function()
{
  # cats body weight & height
  data(cats)

  # plot height as a function of weight
  qplot(cats$B, cats$H)
  # fit a linear regression model
  model <- lm(cats$Hwt ~ cats$Bwt)

  # form predictions
  fitted <- predict(model, cats)
  cats$fitted <- fitted

  # plot the model
  p <-
    ggplot(data=cats) +
      geom_point(aes(x=Bwt, y=Hwt)) +
        geom_line(aes(x=Bwt, y=fitted), colour="red")

  p

  # lets do cross validation

  # first, form five folds
  cats$fold <- as.factor(sample(rep(1:5, length=144)))

  # plot the folds and describe what we are going to do
  p <- ggplot(data=cats, aes(x=Bwt, y=Hwt, colour=fold)) + geom_point()
  p

  # manually run the prediction routine for one fold
  # run the whole study
  cats$pred <- run.cat.study(cats, "Hwt ~ Bwt")

  # compute mean l1
  l1 <- abs(cats$Hwt - cats$pred)
  mean(l1)

  # plot the predictions
  p <-
    ggplot(data=cats, aes(x=Bwt, y=Hwt)) +
    geom_point(colour="black") +
    geom_point(aes(y=pred), colour="red", alpha=0.5)
  p

  # plot error as a funciton of the covariate
  qplot(cats$Bwt, l1)

  # plot error against predicted
  # (note: this can be used when there are many covariates)
  qplot(cats$pred, l1)

  #### multivariate regression

  model <- lm(Hwt ~ Bwt + Sex, cats)
  summary(model)
  model <- lm(Hwt ~ Bwt * Sex, cats)
  summary(model)
  cats$pred2 <- run.cat.study(cats, "Hwt ~ Bwt * Sex")
  l1 <- abs(cats$Hwt - cats$pred)
  mean(l1)
}


run.cat.study <- function(cats, modelstring)
{
  # for each fold
  pred <- numeric(dim(cats)[1])
  for (i in levels(cats$fold))
  {
    out.fold <- which(cats$fold != i)
    in.fold <- which(cats$fold == i)
    model <- lm(modelstring, cats[out.fold,])
    pred[in.fold] <- predict(model, cats[in.fold,])
  }
  pred
}
