do.it <- function()
{
  # load prostate data
  dat <- read.table("~/Desktop/prostate.data")

  # set up training set and test set
  train <- dat[dat$train,]
  test <- dat[!dat$train,]

  # glmnet needs things in this form
  train.x <- as.matrix(train[,1:8])
  train.y <- train[,9]
  test.x <- as.matrix(test[,1:8])
  test.y <- test[,9]

  # compute MLE
  mle.model <- lm(lpsa ~ . - train, data=train)
  # examine MLE coefficients

  # compute ridge regularization path
  ridge.model <- glmnet(x, y, family="gaussian", alpha=0)
  # plot ridge coefficients
  # (if time, make a nicer plot.  there's won't be time.)
  plot(ridge.model)

  # compute lasso regularization path
  lasso.model <- glmnet(x = x, y = y, family = "gaussian", alpha = 1)
  # show lasso coefficients
  lasso.model$beta
  # plot lasso coefficients
  plot(lasso.model)

  # compute training error with MLE
  mle.train <- mean(abs(predict(mle.model, train) - train[,9]))
  # compute training error with ridge
  ridge.train <- glmnet.mean.l1.error(ridge.model, train.x, train.y)
  qplot(ridge.model$lambda, ridge.train)
  # compute training error with lasso
  lasso.train <- glmnet.mean.l1.error(lasso.model, train.x, train.y)
  qplot(lasso.model$lambda, lasso.train)
  # confirm min error between MLE, rdige and lasso is the MLE
  min(lasso.train, ridge.train, mle.train)

  # compute test error with the MLE
  mle.test <- mean(abs(predict(mle.model, test) - test[,9]))
  # compute test error with ridge
  ridge.test <- glmnet.mean.l1.error(ridge.model, test.x, test.y)
  qplot(ridge.model$lambda, ridge.test)
  # compute test error with lasso
  lasso.test <- glmnet.mean.l1.error(lasso.model, test.x, test.y)
  qplot(lasso.model$lambda, lasso.test)

}

glmnet.mean.l1.error <- function(model, x, y)
{
  preds <- predict(model, x, type="response")
  aaply(preds, 2, function (yhat) mean(abs(y - yhat)))
}
